/*
 * File Name : vdce_recv.c
 *
 * Description : This file contains some functions related to the VDCE_RECV function 
 *               which receives data from parent task via socket.
 *
 * Fuunctions  : vdce_recv()
 *               get_data()
 *
 * Programmed by : Ilkyeun Ra, Syracuse University
 *
 * Date        : December 15, 1996
 *
 * Update History
 *
 */
#include "inet.h"
#include "dvm.h"
#include "dvm_msg.h"
#include "vdce.h"

#define External	-1 
#define Remote           0
#define Local            1

extern	struct	task *_dvminfo;
extern	FILE	*jfp;
extern  int 	_cvmsockfd;
extern  VDCE_MSG_HEADER 	 _cvm_header;

/*
 * Function Name : vdce_recv
 *
 * Description   : Receiving all data associated with my task from parent
 *                 task via a socket
 *
 * Input(s)      : port  - logical port number of a task
 *                 bufprt - pointer of receiving message buffer
 *
 * Output(s)     : Receiving data using a socket
 *
*/

vdce_recv(port,bufptr)
VDCE_PORT port;
char      **bufptr;
{
    int         index;
    int         rc;
    int         sockdptr;
    int 	cvmsockfd;
    VDCE_MSG_HEADER   msg_header;
    char	*recv_msg, command[5], msg_data[2048], msg[2048];
    	
/* Checking management channel message */

#ifdef DEBUG
        fprintf(jfp," vdce_recv:before checking cvmport \n ");
	fflush(jfp);
#endif

    cvmsockfd = _cvmsockfd;

    if (readn_avail(&cvmsockfd)) {
	recv_data_using_msg(cvmsockfd, &msg_header, &recv_msg);
#ifdef DEBUG
        fprintf(jfp," from cvm :received %s\n ",recv_msg);
#endif
        decode_command(recv_msg, command, msg_data);

}
	
    index = vdce_get_port(port,VDCERECV);

    if (index == -1) {
	err_ret("vdce_recv: can't find port id in your application");
        rc =0;
        return(rc);
    } 

#ifdef DEBUG
        fprintf(jfp," vdce_recv: checking the type of source\n");
	fflush(jfp);
#endif

    /*
     * Check a type of source
    */
    if (_dvminfo[index].v_type == 0) 
	rc = recv_data_from_file(&_dvminfo[index],bufptr);
    else 
	rc = recv_msg_from_task(&_dvminfo[index],bufptr); 
/*	rc = recv_data_from_task(&_dvminfo[index],bufptr); 
*/

    if (rc ==0) {
#ifdef DEBUG
	fprintf(jfp,"vdce_recv: task %d in %s can't receive data from %s\n",
	_dvminfo[index].d_tkid,_dvminfo[index].d_name,_dvminfo[index].s_name);
#endif
	err_ret("vdce_recv:task can't receive data");
	}
     
#ifdef RECVDEBUG
    {
      int i;
      for(i=0;i<rc;i++) fprintf(jfp,"%c",(*bufptr)[i]
);
      fprintf(jfp,"\n");
      fflush(jfp);
   }
#endif

/*report received data size to CVM */
      sprintf(msg, "%s\n%d\n%d\n%d\n%d\n%d\n", Received_Data_Size, 
	4, /*the number of data lines*/
	_dvminfo[index].user_id, _dvminfo[index].appl_id,
	_dvminfo[index].d_tkid, rc);
      sprintf(_cvm_header.length, "%d",strlen(msg)+HEADERSIZE);
 
#ifdef DEBUG
        fprintf(jfp, "send CVM msg = %s\n", msg);
        fflush(jfp);
#endif
    send_data_using_msg(cvmsockfd, _cvm_header,msg, strlen(msg));

    sockdptr = _dvminfo[index].d_port;
    close(sockdptr);
#ifdef DEBUG
        fprintf(jfp, "vdce_recv done\n");
        fflush(jfp);
#endif
    return(rc);

} /* end vdce_recv */

recv_msg_from_task(taskinfo,bufptr)
struct  task    *taskinfo;
char            **bufptr;
{
    int         rc, i;
    VDCE_MSG_HEADER hmsg;
    int         sockfd;
    char 	*recv_msg; 

    sockfd = taskinfo->d_socket;   /* get destination socket descripto
r */
 
#ifdef DEBUG
    fprintf(jfp,"vdce_recv: task %d's socket fd = %d\n",
	taskinfo->d_tkid,taskinfo->d_socket);
    fflush(jfp);
#endif
 
    if (sockfd == -1) {
#ifdef DEBUG
       fprintf(jfp,"vdce_recv: task %d dose not define socket of port
%d\n",
               taskinfo->d_tkid, taskinfo->d_socket);
#endif
       rc = 0;
       return(rc);
    }
 
 
     rc = recv_data_using_msg(sockfd, &hmsg, &recv_msg);
#ifdef DEBUG
     fprintf(jfp, "received data size = %d\n ", rc);
#endif
#ifdef RECVDEBUG
     fprintf(jfp," received msg ");
     for (i = 0; i< rc ; i++) fprintf(jfp, "%c", recv_msg[i]);
#endif
 
    *bufptr = recv_msg; 

    return(rc);

}

/*
 * Function Name : recv_data_from_task
 *
 * Description   : Receiving data from a task via socket
 *
 * Input(s)      :
 *		   taskinfo - information of task execution environment
 *		   bufptr      - memory buffer pointer
 *
 */
recv_data_from_task(taskinfo,bufptr)
struct	task	*taskinfo;
char		**bufptr;
{
    int         rc;
    VDCE_HEADER	hmsg;
    int		nleft, nread;
    int		sockfd;
    char	*ptr;
    char 	*buf,*savebufptr;
    int		ch;	
 
   /*
    * Get the size of data to be read
   */

    sockfd = taskinfo->d_socket;   /* get destination socket descriptor */

#ifdef DEBUG
    fprintf(jfp,"vdce_recv: task %d's socket fd = %d\n",
		taskinfo->d_tkid,taskinfo->d_socket);
    fflush(jfp);

#endif

    if (sockfd == -1) {
#ifdef DEBUG
       fprintf(jfp,"vdce_recv: task %d dose not define socket of port %d\n",
               taskinfo->d_tkid, taskinfo->d_socket);
#endif
       rc = 0;
       return(rc);
    }


    nleft = sizeof(VDCE_HEADER);
    ptr   = (char *) &hmsg;

#ifdef DEBUG 
   fprintf(jfp,"vdce_recv: Task (%d) will be %d size of msg header\n",
	   taskinfo->d_tkid,nleft);
#endif

    /* loop until whole message header is read */
    while (nleft > 0) {

          nread = read(sockfd, ptr, nleft);

          if ( nread < 0 ) {
#ifdef DEBUG
	     fprintf(jfp,"vdce_recv: task %d read in header\n",
                            taskinfo->d_tkid);
#endif
	     rc = 0;
	     return(0);
          }
          else if (nread == 0) break; /* meet EOF */

          nleft  -= nread;
          ptr    += nread;
    }

    if (nleft) {
#ifdef DEBUG
       fprintf(jfp,"vdce_recv: task %d header is corrupted\n",
	              taskinfo->d_tkid);
#endif
       rc = 0;
       return(rc);
    }

   /*
    * get the real data size and
    * allocate memory buffer
   */
    nleft = hmsg.size;
    buf = (char *) malloc(nleft+1);
    if (buf == NULL) {
#ifdef DEBUG
	fprintf(jfp,"vdce_recv: malloc error for buf\n");
#endif
        return(0);
	}
    bzero(buf,nleft);
    savebufptr = buf;

#ifdef DEBUG
  fprintf(jfp,"vdce_recv: Task(%d) will receive %d size of data\n",
          taskinfo->d_tkid,hmsg.size);
  fflush(jfp);
#endif
    nread = -1;

    while ( nleft > 0) {
	nread=read(sockfd,buf,nleft);
	if (nread < 0) {
#ifdef DEBUG
             fprintf(jfp,"vdce_recv: task %d data is corrupted.\n",
                            taskinfo->d_tkid);
#endif
             rc = 0;
             return(0);
	}
	else if ( nread == 0) break; /* EOF */
	nleft -= nread;
	buf   += nread;
    } 

    if ( nleft ) {
#ifdef DEBUG
       fprintf(jfp, "vdce_rev: task %d data message  corrupted\n", 
		taskinfo->d_tkid);
#endif
       return(0);
    }

    *bufptr = savebufptr ; 
#ifdef DEBUG
      fprintf(jfp,"vdce_recv:Task(%d) has %d size of data:\n",
    taskinfo->d_tkid,strlen(savebufptr));
     fflush(jfp);
#ifdef RECVDEBUG
    {
      int i;
      for(i=0;i<strlen(savebufptr);i++) fprintf(jfp,"%c",savebufptr[i]);
      fprintf(jfp,"\n");
      fflush(jfp);
   }
#endif
#endif
    return(hmsg.size);

} /* end of recv_data_from_task */

/*
 * Function Name : recv_data_from_file
 *
 * Description   : Receiving data from a file of a task
 *
 * Input(s)      :
 *                 taskinfo - information of task execution environment
 *                 buf      - memory buffer pointer
 *
 * Result(s)    : 
 *                 buf - memory buffer where data is stored.
 *                 size of data
 */

recv_data_from_file(taskinfo,bufptr)
struct	task	*taskinfo;
char		**bufptr;
{

    FILE	*fd;
    int         i;
    int         rc;
    int         datasize;
    int	        ch;
    char    	*buf;

    /* check an external file */
    if ( (fd = fopen(taskinfo->v_name,"r")) == NULL) {
#ifdef DEBUG
	  fprintf(jfp,"VDCE_RECV: task %d can't open input file %s\n",
		          taskinfo->d_tkid,taskinfo->v_name);
#endif
	  rc = 0;
	  return(rc);
    }

    /*
     * check size of file and alloc memory buffer
    */

    fseek(fd, 0L, SEEK_END); /* move to end of file */
    datasize = ftell(fd);    /* get size of file */
    fseek(fd, 0L, SEEK_SET); /* rewwind file */

    buf = (char *) calloc(datasize,sizeof(char)); /* alloc memory buffer */
    if (buf == NULL) {
#ifdef DEBUG
	  fprintf(jfp,"VDCE_RECV: task %d an't calloc %s\n",
		          taskinfo->d_tkid,taskinfo->v_name);
#endif
       rc = 0;
       return(rc);
    }
    bzero(buf,datasize);

    i =0;
    while ( i< datasize-1) {
		buf[i] = ch = getc(fd);
		i++;
    }
    fclose(fd);

    rc = strlen(buf);
    *bufptr = buf;

#ifdef RECVDEBUG 
    fprintf(jfp,"vdce_recv(%d):size(%d,%d)%s\n",taskinfo->d_tkid,datasize,rc,buf);
    fflush(jfp);
#endif

    return(rc);
} /* end of recv_data_from_file */


recv_data_using_msg(sockfd, msg_headerpt, bufptr)
int sockfd;
VDCE_MSG_HEADER *msg_headerpt;
char		**bufptr;
{
    int         rc;
    int		nleft, nread, bufsize;
    char	*ptr, stlen[20];
    char 	*buf,*savebufptr;
    int		ch;	
    struct task *taskinfo;

 
   /*
    * Get the size of data to be read
   */



    nleft = sizeof(VDCE_MSG_HEADER);
    ptr   = (char *) msg_headerpt;


    
    /* loop until whole message header is read */
    while (nleft > 0) {

#ifdef DEBUG
      fprintf(jfp,"recv_data_using_msg : before read vdce_header\n");
      fflush(jfp);
#endif

          nread = read(sockfd, ptr, nleft);

          if ( nread < 0 ) {
	     err_ret("recv_msg: task  read in header");
	     rc = 0;
	     return(0);
          }
          else if (nread == 0) break; /* meet EOF */

          nleft  -= nread;
          ptr    += nread;
    }

    if (nleft) {
       err_ret("recv_msg: task  header is corrupted");
       rc = 0;
       return(rc);
    }

#ifdef DEBUG
      fprintf(jfp,"recv_data_using_msg : done header read %s\n", ptr);
      fflush(jfp);
#endif
   /*
    * get the real data size and
    * allocate memory buffer
   */
    strncpy(stlen, msg_headerpt->length, 8);

    nleft = atoi(stlen);
    nleft -= HEADERSIZE;
    bufsize = nleft;
    buf = (char *) malloc(nleft+1);
    if (buf == NULL) {
#ifdef DEBUG
	  fprintf(jfp,"VDCE_RECV: task %d an't calloc %s\n",
		          taskinfo->d_tkid,taskinfo->v_name);
#endif
       rc = 0;
       return(rc);
    }
    bzero(buf,nleft+1);
    savebufptr = buf;

#ifdef DEBUG
  fprintf(jfp,"recv_msg: Task will receive %d size of data %s\n",
          nleft, msg_headerpt->H_name);
  fflush(jfp);
#endif
    nread = -1;

    while ( nleft > 0) {
	nread=read(sockfd,buf,nleft);
	if (nread < 0) {
#ifdef DEBUG
             fprintf(jfp,"vdce_recv: task %d data is corrupted.\n",
                            taskinfo->d_tkid);
#endif
             rc = 0;
             return(0);
	}
	else if ( nread == 0) break; /* EOF */
	nleft -= nread;
	buf   += nread;
    } 

    if ( nleft ) {
       err_ret( "vdce_rev: task  data message  corrupted\n" );
       return(0);
    }

    *bufptr=  savebufptr ; 

#ifdef RECVDEBUG
      fprintf(jfp,"recv_dat_using_msg:Task() has %d size of data %s\n",
    bufsize, *bufptr);
     fflush(jfp);
/* test for matrix only 
	{
typedef struct Matrix_struct {
  int   matrix_size;
  float matrix[16][16];
} matrix;

      matrix *show_matrix;
      int i, j;

      show_matrix = (matrix *)savebufptr;

      fprintf(jfp, "RECV: matrix size : %d\n", show_matrix->matrix_size);
  for ( i = 0 ; i < 16 ; i++ ) {
    for ( j = 0 ; j < 16 ; j++ ) {
      fprintf(jfp, "%f  ", show_matrix->matrix[i][j]);
      }
    fprintf(jfp, "\n");
    }

   }
*/
#endif
#ifdef DEBUG
      fprintf(jfp,"recv_dat_using_msg:Task() has %d size of data\n",bufsize);
      fprintf(jfp,"\nbefore completing recv_data_using_msg msg_headerpt->length=%s\n ", msg_headerpt->length);
     fflush(jfp);
#endif
    return(bufsize);

} /* end of recv_data_using_msg */
